// 002 u_c_ib_blob_script_extractor
// 22 oct 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_ib_blob_script_extractor;
  interface
    uses Classes, IbDatabase, u_c_basic_object;

    type c_ib_blob_script_extractor= class(c_basic_object)
                                       m_c_ibdatabase_ref: tibDatabase;
                                       m_c_result_script: tStringList;

                                       Constructor create_ib_blob_script_extractor(p_name: String;
                                           p_c_ibdatabase_ref: tIbDatabase);
                                       procedure extract_script(p_table_name: String);

                                       Destructor Destroy; Override;
                                     end; // c_ib_blob_script_extractor

  implementation
    uses SysUtils, Db, IbQuery, u_c_display
        , u_strings
        , u_ibx
        , u_db
        ;

    // -- c_ib_blob_script_extractor

    Constructor c_ib_blob_script_extractor.create_ib_blob_script_extractor(p_name: String;
        p_c_ibdatabase_ref: tIbDatabase);
      begin
        Inherited create_basic_object(p_name);

        m_c_ibdatabase_ref:= p_c_ibdatabase_ref;
      end; // create_ib_blob_script_extractor

    procedure c_ib_blob_script_extractor.extract_script(p_table_name: String);
      var l_c_ibquery: tIbQuery;

      procedure open_table;
        begin
          f_open_ib_database('', m_c_ibdatabase_ref);

          l_c_ibquery:= tIbQuery.Create(Nil);
          with l_c_ibquery do
          begin
            Database:= m_c_ibdatabase_ref;

            open_ibquery(l_c_ibquery, 'SELECT * FROM '+ p_table_name);
          end; // with l_c_ibquery
        end; // open_table

      var l_insert_into: String;

      procedure build_start_insert_into_request;
        var l_field_def_index: Integer;
            l_field_name_list: String;
        begin
          l_field_name_list:= '';

          with l_c_ibquery, FieldDefs do
            for l_field_def_index:= 0 to Count- 1 do
              with FieldDefs[l_field_def_index] do
              begin
                l_field_name_list:= l_field_name_list+ Name;
                if l_field_def_index< Count- 1
                  then l_field_name_list:= l_field_name_list+ ', ';
              end;
          l_insert_into:= 'INSERT INTO '+ p_table_name+ ' ('+ l_field_name_list+ ' ) ';

          display(l_insert_into);
        end; // build_start_insert_into_request

      procedure generate_script;

        procedure trace_script(p_text: String);
          begin
          end; // trace_script

        procedure generate_the_script;
            // -- generate the values for the current line
          var l_field_index: Integer;
              l_values: String;

              // l_c_memory_stream: tMemoryStream;
              l_c_string_stream: tStringStream;
          begin
            l_values:= '';

            with l_c_ibquery, FieldDefs do
              for l_field_index:= 0 to Count- 1 do
              begin
                case FieldDefs[l_field_index].DataType of
                  ftBlob :
                    begin
                      trace_script('  ftBlob (binary)');
                      // l_c_field_value.m_c_stream:= tMemoryStream.Create;
                      // display_not_implemented('generate_the_script');
                    end;
                  ftMemo :
                    begin
                      trace_script('  ftMemo '+ Name);

                      l_c_string_stream:= tStringStream.Create('');
                      (Fields[l_field_index] as tMemoField).SaveToStream(l_c_string_stream);
                      display('%'+ f_add_return_line_feed(l_c_string_stream.DataString)+ '%');

                      // -- todo: handle inside quotes ?
                      l_values:= l_values+ QuotedStr(l_c_string_stream.DataString);
                      l_c_string_stream.Free;
                    end;

                  ftString :
                      begin
                        // -- varying text
                        trace_script('  ftString');
                        // l_values:= l_values+ QuotedString''''+ Fields[l_field_index].AsString+ '''';
                        l_values:= l_values+ QuotedStr(Fields[l_field_index].AsString);
                      end;
                  ftSmallint :
                      begin
                        // -- short
                        trace_script('  ftSmallInt');
                        l_values:= l_values+ Fields[l_field_index].AsString;
                      end;
                  ftFloat :
                      begin
                        // -- double
                        trace_script('  ftFloat');
                        l_values:= l_values+ Fields[l_field_index].AsString;
                      end;

                  else
                    display(f_fieldtype_name(FieldDefs[l_field_index].DataType));
                  // display_not_implemented('generate_the_script');
                end; // case

                if l_field_index< Count- 1
                  then l_values:= l_values+ ', ';

              end; // with l_c_ibquery, FieldDefs, for l_field_index

            display('  |'+ l_values+ '|');

            with m_c_result_script do
            begin
              Add(l_insert_into);
              // -- append VALUES AND a script terminator
              Add('  VALUES ('+ l_values+ ')' + ';');
            end;
          end; // generate_the_script

        begin // generate_script
          with l_c_ibquery do
            while not Eof do
            begin
              generate_the_script;
              Next;
            end; // with l_c_ibquery, while not eof
        end; // generate_script

      begin // extract_script
        m_c_result_script.Free;
        m_c_result_script:= tStringList.Create;

        open_table;
        // -- get the table fields
        build_start_insert_into_request;
        generate_script;
      end; // extract_script

    Destructor c_ib_blob_script_extractor.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_ib_blob_script_extractor
    end. // u_c_ib_blob_script_extractor


