// 002 u_ibx
// 16 oct 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_ibx;
  interface
    uses Classes, IbDatabase, IbSql, IbQuery
       ;

    function f_create_ib_database(p_database_name, p_user, p_password: String; p_c_ib_database: tIbDatabase): Boolean;
    procedure initialize_ib_database(p_database_name, p_user, p_password: String; p_c_ib_database: tIbDatabase);
    procedure display_ib_database_name_info(p_c_ib_database: tIbDatabase);

    function f_open_ib_database(p_message: String; p_c_ib_database: tIbDatabase): Boolean;
    function f_execute_ibsql(p_c_ibqsl: tIbSql; p_request: String): Boolean;
    procedure open_ibquery(p_c_ibquery: tIbQuery; p_select_request: String);

    function f_c_table_names(p_c_ib_database: tIbDatabase): tStringList;
    function f_c_field_info_ibquery(p_c_ib_database: tIbDatabase; p_table_name: String): tIbQuery;

  implementation
    uses SysUtils
        , u_file
        , u_c_display
        ;

    function f_create_ib_database(p_database_name, p_user, p_password: String; p_c_ib_database: tIbDatabase): Boolean;
      begin
        with p_c_ib_database, Params do
        begin
          Result:= False;
          Connected:= False;

          If FileExists(p_database_name)
            Then begin
                f_erase(p_database_name);
              end;

          DatabaseName:= p_database_name;
          SqlDialect:= 3;
          Clear;

          Add('USER "'+ p_user+ '"');
          Add('PASSWORD "'+ p_password+ '"');
          Add('PAGE_SIZE 4096');

          Try
            CreateDatabase;
            Result:= True;
          except
            on e: Exception do
              display_bug_stop('*** err '+ e.Message);
          end; // try
        end; // with IbDatabase1, Params
      end; // f_create_ib_database

    procedure initialize_ib_database(p_database_name, p_user, p_password: String; p_c_ib_database: tIbDatabase);
      begin
        with p_c_ib_database do
        begin
          If Connected
            then Close;

          DatabaseName:= p_database_name;
          Params.Clear;
          Params.Add('user_name='+ p_user);
          Params.Add('password='+ p_password);
        end;
      end; // initialize_ib_database

    procedure display_ib_database_name_info(p_c_ib_database: tIbDatabase);
      begin
        with p_c_ib_database do
        begin
          display('name >'+ DatabaseName+ '<');
          display('dia  >'+ IntToStr(SqlDialect));
          display('params >'+ Params.Text+ '|');
        end;
      end; // display_ib_database_name_info

    function f_open_ib_database(p_message: String; p_c_ib_database: tIbDatabase): Boolean;
      begin
        Result:= False;

        if p_message<> ''
          then display('> '+ p_message);

        with p_c_ib_database do
        begin
          Try
            if not Connected
              then Open;
            Result:= True;  
          Except
            on e: exception do
              display('ib_db_open_err '+ e.Message);
          end; // try except
        end; // with p_c_ib_database

        if p_message<> ''
          then display('< '+ p_message);
      end; // f_open_ib_database

    function f_execute_ibsql(p_c_ibqsl: tIbSql; p_request: String): Boolean;
      begin
        // display('> execute_query');
        Result:= False;

        with p_c_ibqsl do
        begin
          // display_strings_only(p_request);

          if True // m_exec_sql
            then
              try
                f_open_ib_database('', DataBase);

                Sql.Text:= p_request;

                if DataBase.DefaultTransaction.InTransaction
                  then DataBase.DefaultTransaction.Commit;
                DataBase.DefaultTransaction.StartTransaction;
                ExecQuery;
                DataBase.DefaultTransaction.Commit;

                // m_explain:= '  ok '+ p_title;
                // display(m_explain);
                Result:= True;
              except
                on e: Exception do
                  begin
                    // m_explain:= '*** pb_'+ p_title+ ' '+ e.Message;
                    display_bug_stop('*** pb_'+ e.Message);
                  end;
              end // try except
        end; // with m_c_ib_sql_ref
      end; // f_execute_ibsql

    procedure open_ibquery(p_c_ibquery: tIbQuery; p_select_request: String);
      begin
        with p_c_ibquery, Sql do
        begin
          Close;
          Clear;
          Text:= p_select_request;
          try
            Open;
          except
            on e: exception do
              display('ibq_err '+ e.Message);
          end; // try except
        end; // open_ibquery
      end; // open_ibquery

    function f_c_table_names(p_c_ib_database: tIbDatabase): tStringList;
      const k_select_table_names=
                  'SELECT RDB$RELATION_NAME'
                + '  FROM RDB$RELATIONS'
                + '  WHERE '
                + '      ('
                + '         (RDB$SYSTEM_FLAG = 0) '
                + '        OR'
                + '          (RDB$SYSTEM_FLAG IS NULL)'
                + '       )'
                + '     AND'
                + '      (RDB$VIEW_SOURCE IS NULL)'
                + '  ORDER BY RDB$RELATION_NAME'
                ;
      var l_c_ibquery: tIbQuery;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibquery:= tIbQuery.Create(Nil);
        with l_c_ibquery do
        begin
          Database:= p_c_ib_database;
          open_ibquery(l_c_ibquery, k_select_table_names);

          Result:= tStringList.Create;
          While not Eof do
          begin
            Result.Add(Fields[0].AsString);
            Next;
          end; // While not Eof

          Close;
          Free;
        end; // with tIbQuery
      end; // f_c_table_names

    function f_c_field_info_ibquery(p_c_ib_database: tIbDatabase; p_table_name: String): tIbQuery;
      const k_select_field_info=
                  'select r.rdb$field_name, '
                + '    t.rdb$type_name, '
                + '    f.rdb$field_length, '
                + '    f.rdb$field_sub_type '
                + '  from rdb$relation_fields r, rdb$types t, rdb$fields f '
                + '  where r.rdb$relation_name= ''%s'' ' 
                + '    and f.rdb$field_name=r.rdb$field_source '
                + '    and t.rdb$field_name= ''RDB$FIELD_TYPE'' '
                + '    and f.rdb$field_type=t.rdb$type'
                ;
      var l_request: String;
      begin
        f_open_ib_database('', p_c_ib_database);

        Result:= tIbQuery.Create(Nil);
        with Result do
        begin
          Database:= p_c_ib_database;
          l_request:= Format(k_select_field_info, [p_table_name]);
          open_ibquery(Result, l_request);
        end; // with Result
      end; // f_c_field_info_ibquery


    begin // u_zzz
    end. // u_zzz
